﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

// https://github.com/eclipse/paho.mqtt.m2mqtt
using uPLibrary.Networking.M2Mqtt;
using uPLibrary.Networking.M2Mqtt.Messages;
using System.Net;

using Newtonsoft.Json;

using eSkin4Common;

namespace eSkin4Bitalino
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// BITalino device handler
        /// </summary>
        /// <remarks>
        /// See: https://github.com/BITalinoWorld/dotNet-api
        /// </remarks>
        private static bool bitalinoIsEnabled = bool.Parse(ConfigurationManager.AppSettings["bitalinoIsEnabled"]);
        List<BitalinoHandler> bitalinoHandlers = new List<BitalinoHandler>();
        private static int bitalinoSampleRate = int.Parse(ConfigurationManager.AppSettings["bitalinoSampleRate"]);

        private static bool mqttIsEnabled = bool.Parse(ConfigurationManager.AppSettings["mqttIsEnabled"]);
        string mqttBrokerHostname = ConfigurationManager.AppSettings["mqttBrokerHostname"];
        MqttHandler mqttHandler = null;

        private static bool oscIsEnabled = bool.Parse(ConfigurationManager.AppSettings["oscIsEnabled"]);
        string oscAddressSetting = ConfigurationManager.AppSettings["oscAddress"];
        string oscPortSetting = ConfigurationManager.AppSettings["oscPort"];
        OscHandler oscHandler = null;

        /// <summary>
        /// Main program window .ctor
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            // Connect to console
            TextBoxConsole.RedirectConsoleToTextBox(messageConsole);
            Console.WriteLine("eSkin4Sensors: Bitalino started");

            // Get default configuration and create handlers
            if (mqttIsEnabled)
            {
                if (!string.IsNullOrEmpty(mqttBrokerHostname))
                {
                    mqttBroker.Text = mqttBrokerHostname;
                }

                List<Label> mqttLabels = new List<Label>();
                mqttLabels.Add(this.mqttMessages);

                this.mqttHandler = new MqttHandler("Bitalino", mqttBrokerHostname, this.Dispatcher, mqttLabels);
            }

            if (oscIsEnabled)
            {
                if (!string.IsNullOrEmpty(oscAddressSetting))
                {
                    oscAddress.Text = oscAddressSetting;
                }

                if (!string.IsNullOrEmpty(oscPortSetting))
                {
                    oscPort.Text = oscPortSetting;
                }


                List<Label> oscLabels = new List<Label>();
                oscHandler = new OscHandler(oscAddressSetting, oscPortSetting, this.Dispatcher, oscLabels);
            }

            if (bitalinoIsEnabled)
            {
                // Initialize addresses
                string bitalino1Address = ConfigurationManager.AppSettings["bitalino1Address"];
                string bitalino1Name = ConfigurationManager.AppSettings["bitalino1Name"];
                this.bitalino1Address.Text = bitalino1Address;
                if (!string.IsNullOrEmpty(bitalino1Address))
                {
                    this.bitalino1Address.Text = bitalino1Address;
                }

                string bitalino2Address = ConfigurationManager.AppSettings["bitalino2Address"];
                string bitalino2Name = ConfigurationManager.AppSettings["bitalino2Name"];
                this.bitalino2Name.Content = bitalino2Name;
                if (!string.IsNullOrEmpty(bitalino2Address))
                {
                    this.bitalino2Address.Text = bitalino2Address;
                }

                string bitalino3Address = ConfigurationManager.AppSettings["bitalino3Address"];
                string bitalino3Name = ConfigurationManager.AppSettings["bitalino3Name"];
                this.bitalino3Address.Text = bitalino3Address;
                if (!string.IsNullOrEmpty(bitalino3Address))
                {
                    this.bitalino3Address.Text = bitalino3Address;
                }

                // Initialize UI labels
                List<Label> bitalinoLabels1 = new List<Label>();
                bitalinoLabels1.Add(this.bitalino1Name);
                bitalinoLabels1.Add(this.bitalino1D1);
                bitalinoLabels1.Add(this.bitalino1D2);
                bitalinoLabels1.Add(this.bitalino1D3);
                bitalinoLabels1.Add(this.bitalino1D4);
                bitalinoLabels1.Add(this.bitalino1A1);
                bitalinoLabels1.Add(this.bitalino1A2);
                bitalinoLabels1.Add(this.bitalino1A3);
                bitalinoLabels1.Add(this.bitalino1A4);
                bitalinoLabels1.Add(this.bitalino1A5);
                bitalinoLabels1.Add(this.bitalino1A6);
                List<Label> bitalinoLabels2 = new List<Label>();
                bitalinoLabels2.Add(this.bitalino2Name);
                bitalinoLabels2.Add(this.bitalino2D1);
                bitalinoLabels2.Add(this.bitalino2D2);
                bitalinoLabels2.Add(this.bitalino2D3);
                bitalinoLabels2.Add(this.bitalino2D4);
                bitalinoLabels2.Add(this.bitalino2A1);
                bitalinoLabels2.Add(this.bitalino2A2);
                bitalinoLabels2.Add(this.bitalino2A3);
                bitalinoLabels2.Add(this.bitalino2A4);
                bitalinoLabels2.Add(this.bitalino2A5);
                bitalinoLabels2.Add(this.bitalino2A6);
                List<Label> bitalinoLabels3 = new List<Label>();
                bitalinoLabels3.Add(this.bitalino3Name);
                bitalinoLabels3.Add(this.bitalino3D1);
                bitalinoLabels3.Add(this.bitalino3D2);
                bitalinoLabels3.Add(this.bitalino3D3);
                bitalinoLabels3.Add(this.bitalino3D4);
                bitalinoLabels3.Add(this.bitalino3A1);
                bitalinoLabels3.Add(this.bitalino3A2);
                bitalinoLabels3.Add(this.bitalino3A3);
                bitalinoLabels3.Add(this.bitalino3A4);
                bitalinoLabels3.Add(this.bitalino3A5);
                bitalinoLabels3.Add(this.bitalino3A6);

                BitalinoHandler bitalinoHandler1 = new BitalinoHandler(
                    0, 
                    bitalino1Address,
                    bitalino1Name,
                    bitalinoSampleRate, 
                    this.mqttHandler, 
                    this.oscHandler,
                    this.Dispatcher, 
                    bitalinoLabels1);
                bitalinoHandlers.Add(bitalinoHandler1);

                BitalinoHandler bitalinoHandler2 = new BitalinoHandler(
                    1, 
                    bitalino2Address,
                    bitalino2Name,
                    bitalinoSampleRate, 
                    this.mqttHandler,
                    this.oscHandler,
                    this.Dispatcher, 
                    bitalinoLabels2);
                bitalinoHandlers.Add(bitalinoHandler2);

                BitalinoHandler bitalinoHandler3 = new BitalinoHandler(
                    2, 
                    bitalino3Address,
                    bitalino3Name,
                    bitalinoSampleRate, 
                    this.mqttHandler, 
                    this.oscHandler,
                    this.Dispatcher, 
                    bitalinoLabels3);
                bitalinoHandlers.Add(bitalinoHandler3);

                // Set source
                this.mqttSource.Content = Environment.MachineName;

                // Show topic
                this.mqttTopic1.Content = BitalinoHandler.BitalinoTopic1;
            }
        }

        #region Window handlers

        /// <summary>
        /// Startup program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (mqttIsEnabled)
            {
                // Open MQTT 
                bool mqttIsConnected = mqttHandler.OpenMqtt();
                mqttStatusIndicator.Fill = mqttIsConnected ? Brushes.Green : Brushes.Red;
            }

            if (oscIsEnabled)
            {
                bool oscIsConnected = oscHandler.OpenOsc();
                oscStatusIndicator.Fill = oscIsConnected ? Brushes.Green : Brushes.Red;
            }

            if (bitalinoIsEnabled)
            {
                // Open BITalinos
                bool bitalino1IsConnected = bitalinoHandlers[0].OpenBitalino();
                bitalino1StatusIndicator.Fill = bitalino1IsConnected ? Brushes.Green : Brushes.Red;
                bool bitalino2IsConnected = bitalinoHandlers[1].OpenBitalino();
                bitalino2StatusIndicator.Fill = bitalino2IsConnected ? Brushes.Green : Brushes.Red;
                bool bitalino3IsConnected = bitalinoHandlers[2].OpenBitalino();
                bitalino3StatusIndicator.Fill = bitalino3IsConnected ? Brushes.Green : Brushes.Red;
            }
        }

        /// <summary>
        /// Shutdown program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closed(object sender, EventArgs e)
        {
            if (mqttIsEnabled)
            {
                // Close MQTT 
                mqttHandler.CloseMqtt();
                mqttStatusIndicator.Fill = Brushes.LightGray;
            }

            if (oscIsEnabled)
            {
                // Close OSC
                oscHandler.CloseOsc();
                oscStatusIndicator.Fill = Brushes.LightGray;
            }

            if (bitalinoIsEnabled)
            {
                // Close Bitalinos
                bitalinoHandlers[0].CloseBitalino();
                bitalino1StatusIndicator.Fill = Brushes.LightGray;
                bitalinoHandlers[1].CloseBitalino();
                bitalino2StatusIndicator.Fill = Brushes.LightGray;
                bitalinoHandlers[2].CloseBitalino();
                bitalino3StatusIndicator.Fill = Brushes.LightGray;
            }
        }

        #endregion

        private void bitalino1Reconnect_Click(object sender, RoutedEventArgs e)
        {
            if (bitalinoIsEnabled)
            {
                // Close Bitalino 1
                bitalinoHandlers[0].CloseBitalino();
                bitalino1StatusIndicator.Fill = Brushes.LightGray;

                // Update bitalino address from UI
                bitalinoHandlers[0].BitalinoAddress = bitalino1Address.Text;

                // Open BITalino 1
                bool bitalino1IsConnected = bitalinoHandlers[0].OpenBitalino();
                bitalino1StatusIndicator.Fill = bitalino1IsConnected ? Brushes.Green : Brushes.Red;
            }
        }

        private void bitalino2Reconnect_Click(object sender, RoutedEventArgs e)
        {
            if (bitalinoIsEnabled)
            {
                // Close Bitalino 2
                bitalinoHandlers[1].CloseBitalino();
                bitalino2StatusIndicator.Fill = Brushes.LightGray;

                // Update bitalino address from UI
                bitalinoHandlers[1].BitalinoAddress = bitalino2Address.Text;

                // Open BITalino 2
                bool bitalino2IsConnected = bitalinoHandlers[1].OpenBitalino();
                bitalino2StatusIndicator.Fill = bitalino2IsConnected ? Brushes.Green : Brushes.Red;
            }
        }

        private void bitalino3Reconnect_Click(object sender, RoutedEventArgs e)
        {
            if (bitalinoIsEnabled)
            {
                // Close Bitalino 3
                bitalinoHandlers[2].CloseBitalino();
                bitalino3StatusIndicator.Fill = Brushes.LightGray;

                // Update bitalino address from UI
                bitalinoHandlers[2].BitalinoAddress = bitalino3Address.Text;

                // Open BITalino 3
                bool bitalino3IsConnected = bitalinoHandlers[2].OpenBitalino();
                bitalino3StatusIndicator.Fill = bitalino3IsConnected ? Brushes.Green : Brushes.Red;
            }
        }

        private void mqttReconnect_Click(object sender, RoutedEventArgs e)
        {
            if (mqttIsEnabled)
            {
                // Close MQTT 
                mqttHandler.CloseMqtt();
                mqttStatusIndicator.Fill = Brushes.LightGray;

                // Update hostname from UI
                mqttHandler.BrokerHostname = mqttBroker.Text;

                // Open MQTT 
                bool mqttIsConnected = mqttHandler.OpenMqtt();
                mqttStatusIndicator.Fill = mqttIsConnected ? Brushes.Green : Brushes.Red;
            }
        }

        private void oscReconnect_Click(object sender, RoutedEventArgs e)
        {
            if (oscIsEnabled)
            {
                // Close OSC
                oscHandler.CloseOsc();
                oscStatusIndicator.Fill = Brushes.LightGray;

                // Update address and port from UI
                oscHandler.Address = oscAddress.Text;
                oscHandler.Port = oscPort.Text;

                // Open OSC
                bool oscIsConnected = oscHandler.OpenOsc();
                oscStatusIndicator.Fill = oscIsConnected ? Brushes.Green : Brushes.Red;
            }
        }


        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            mqttHandler.SaveMessages = true;
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            mqttHandler.SaveMessages = false;
        }

        /// <summary>
        /// Configuration button click: opens Notepad to edit App.config
        /// </summary>
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ConfigurationFile.Edit();
        }
    }
}

