﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

// https://github.com/eclipse/paho.mqtt.m2mqtt
using uPLibrary.Networking.M2Mqtt;
using uPLibrary.Networking.M2Mqtt.Messages;
using System.Net;

using Newtonsoft.Json;

using eSkin4Common;
using System.Windows.Threading;

namespace eSkin4MessageSimulator
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MqttHandler mqttHandler = null;

        private DispatcherTimer sendDispatcherTimer = null;

        private bool autoSend = false;

        public MainWindow()
        {
            InitializeComponent();

            string mqttBrokerHostname = ConfigurationManager.AppSettings["mqttBrokerHostname"];
            if (!string.IsNullOrEmpty(mqttBrokerHostname))
            {
                mqttBroker.Text = mqttBrokerHostname;
            }

            string mqttTopicDefault = ConfigurationManager.AppSettings["mqttTopicDefault"];
            mqttTopic.Text = mqttTopicDefault;
            string mqttMessageDefault = ConfigurationManager.AppSettings["mqttMessageDefault"];
            mqttMessage.Text = mqttMessageDefault.Replace("'", "\"");

            List<Label> mqttLabels = new List<Label>();
            mqttLabels.Add(this.mqttMessages);

            this.mqttHandler = new MqttHandler("Simulator", mqttBrokerHostname, this.Dispatcher, mqttLabels);

            //  DispatcherTimer setup
            sendDispatcherTimer = new System.Windows.Threading.DispatcherTimer();
            sendDispatcherTimer.Tick += new EventHandler(SendDispatcherTimerTick);
            sendDispatcherTimer.Interval = new TimeSpan(0, 0, 1);
            sendDispatcherTimer.Start();
        }

        #region Window handlers

        /// <summary>
        /// Startup program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // Open MQTT 
            bool mqttIsConnected = mqttHandler.OpenMqtt();
            mqttStatusIndicator.Fill = mqttIsConnected ? Brushes.Green : Brushes.Red;
        }

        /// <summary>
        /// Shutdown program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closed(object sender, EventArgs e)
        {
            // Close MQTT 
            mqttHandler.CloseMqtt();
            mqttStatusIndicator.Fill = Brushes.LightGray;
        }

        #endregion

        private void mqttReconnect_Click(object sender, RoutedEventArgs e)
        {
            // Close MQTT 
            mqttHandler.CloseMqtt();
            mqttStatusIndicator.Fill = Brushes.LightGray;

            // Update hostname from UI
            mqttHandler.BrokerHostname = mqttBroker.Text;

            // Open MQTT 
            bool mqttIsConnected = mqttHandler.OpenMqtt();
            mqttStatusIndicator.Fill = mqttIsConnected ? Brushes.Green : Brushes.Red;
        }

        private void mqttMessageSend_Click(object sender, RoutedEventArgs e)
        {
            SendMQTTMessage();
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            autoSend = true;
        }

        private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            autoSend = false;
        }

        // System.Windows.Threading.DispatcherTimer.Tick handler
        //
        // Sends a MQTT messages (when enabled).
        private void SendDispatcherTimerTick(object sender, EventArgs e)
        {

            if (autoSend)
            {
                SendMQTTMessage();
            }
        }

        /// <summary>
        /// Sends a MQTT messages.
        /// </summary>
        private void SendMQTTMessage()
        {
            if (mqttHandler != null &&
                mqttHandler.IsConnected)
            {
                string topic = mqttTopic.Text;
                string message = mqttMessage.Text;
                if (!string.IsNullOrEmpty(topic) &&
                    !string.IsNullOrEmpty(message))
                    mqttHandler.SendMessage(topic, message);
            }
        }
    }
}
